# Copyright (C) 2015-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

import datetime


from swh.model import hashutil

from swh.web.common import converters
from swh.web.tests.testcase import WebTestCase


class ConvertersTestCase(WebTestCase):

    def test_fmap(self):
        self.assertEqual([2, 3, None, 4],
                         converters.fmap(lambda x: x+1, [1, 2, None, 3]))
        self.assertEqual([11, 12, 13],
                         list(converters.fmap(lambda x: x+10,
                                              map(lambda x: x, [1, 2, 3]))))
        self.assertEqual({'a': 2, 'b': 4},
                         converters.fmap(lambda x: x*2, {'a': 1, 'b': 2}))
        self.assertEqual(100,
                         converters.fmap(lambda x: x*10, 10))
        self.assertEqual({'a': [2, 6], 'b': 4},
                          converters.fmap(lambda x: x*2, {'a': [1, 3], 'b': 2})) # noqa

        self.assertIsNone(converters.fmap(lambda x: x, None))

    def test_from_swh(self):
        some_input = {
            'a': 'something',
            'b': 'someone',
            'c': b'sharp-0.3.4.tgz',
            'd': hashutil.hash_to_bytes(
                'b04caf10e9535160d90e874b45aa426de762f19f'),
            'e': b'sharp.html/doc_002dS_005fISREG.html',
            'g': [b'utf-8-to-decode', b'another-one'],
            'h': 'something filtered',
            'i': {'e': b'something'},
            'j': {
                'k': {
                    'l': [b'bytes thing', b'another thingy', b''],
                    'n': 'dont care either'
                },
                'm': 'dont care'
            },
            'o': 'something',
            'p': b'foo',
            'q': {'extra-headers': [['a', b'intact']]},
            'w': None,
            'r': {'p': 'also intact',
                  'q': 'bar'},
            's': {
                'timestamp': 42,
                'offset': -420,
                'negative_utc': None,
            },
            's1': {
                'timestamp': {'seconds': 42, 'microseconds': 0},
                'offset': -420,
                'negative_utc': None,
            },
            's2': datetime.datetime(
                2013, 7, 1, 20, 0, 0,
                tzinfo=datetime.timezone.utc),
            't': None,
            'u': None,
            'v': None,
            'x': None,
        }

        expected_output = {
            'a': 'something',
            'b': 'someone',
            'c': 'sharp-0.3.4.tgz',
            'd': 'b04caf10e9535160d90e874b45aa426de762f19f',
            'e': 'sharp.html/doc_002dS_005fISREG.html',
            'g': ['utf-8-to-decode', 'another-one'],
            'i': {'e': 'something'},
            'j': {
                'k': {
                    'l': ['bytes thing', 'another thingy', '']
                }
            },
            'p': 'foo',
            'q': {'extra-headers': [['a', 'intact']]},
            'w': {},
            'r': {'p': 'also intact',
                  'q': 'bar'},
            's': '1969-12-31T17:00:42-07:00',
            's1': '1969-12-31T17:00:42-07:00',
            's2': '2013-07-01T20:00:00+00:00',
            'u': {},
            'v': [],
            'x': None,
        }

        actual_output = converters.from_swh(
            some_input,
            hashess={'d', 'o', 'x'},
            bytess={'c', 'e', 'g', 'l'},
            dates={'s', 's1', 's2'},
            blacklist={'h', 'm', 'n', 'o'},
            removables_if_empty={'t'},
            empty_dict={'u'},
            empty_list={'v'},
            convert={'p', 'q', 'w'},
            convert_fn=converters.convert_revision_metadata)

        self.assertEqual(expected_output, actual_output)

    def test_from_swh_edge_cases_do_no_conversion_if_none_or_not_bytes(self):
        some_input = {
            'a': 'something',
            'b': None,
            'c': 'someone',
            'd': None,
            'e': None
        }

        expected_output = {
            'a': 'something',
            'b': None,
            'c': 'someone',
            'd': None,
            'e': None
        }

        actual_output = converters.from_swh(some_input,
                                            hashess={'a', 'b'},
                                            bytess={'c', 'd'},
                                            dates={'e'})

        self.assertEqual(expected_output, actual_output)

    def test_from_swh_edge_cases_convert_invalid_utf8_bytes(self):
        some_input = {
            'a': 'something',
            'b': 'someone',
            'c': b'a name \xff',
            'd': b'an email \xff',
        }

        expected_output = {
            'a': 'something',
            'b': 'someone',
            'c': 'a name \\xff',
            'd': 'an email \\xff',
            'decoding_failures': ['c', 'd']
        }

        actual_output = converters.from_swh(some_input,
                                            hashess={'a', 'b'},
                                            bytess={'c', 'd'})
        for v in ['a', 'b', 'c', 'd']:
            self.assertEqual(expected_output[v], actual_output[v])
        self.assertEqual(len(expected_output['decoding_failures']),
                         len(actual_output['decoding_failures']))
        for v in expected_output['decoding_failures']:
            self.assertTrue(v in actual_output['decoding_failures'])

    def test_from_swh_empty(self):
        # when
        self.assertEqual({}, converters.from_swh({}))

    def test_from_swh_none(self):
        # when
        self.assertIsNone(converters.from_swh(None))

    def test_from_origin(self):
        # given
        origin_input = {
            'id': 9,
            'type': 'ftp',
            'url': 'rsync://ftp.gnu.org/gnu/octave',
        }

        expected_origin = {
            'id': 9,
            'type': 'ftp',
            'url': 'rsync://ftp.gnu.org/gnu/octave',
        }

        # when
        actual_origin = converters.from_origin(origin_input)

        # then
        self.assertEqual(actual_origin, expected_origin)

    def test_from_origin_visit(self):
        snap_hash = 'b5f0b7f716735ebffe38505c60145c4fd9da6ca3'

        for snap in [snap_hash, None]:
            # given
            visit = {
                'date': {
                    'timestamp': datetime.datetime(
                        2015, 1, 1, 22, 0, 0,
                        tzinfo=datetime.timezone.utc).timestamp(),
                    'offset': 0,
                    'negative_utc': False,
                },
                'origin': 10,
                'visit': 100,
                'metadata': None,
                'status': 'full',
                'snapshot': hashutil.hash_to_bytes(snap) if snap else snap,
            }

            expected_visit = {
                'date': '2015-01-01T22:00:00+00:00',
                'origin': 10,
                'visit': 100,
                'metadata': {},
                'status': 'full',
                'snapshot': snap_hash if snap else snap
            }

            # when
            actual_visit = converters.from_origin_visit(visit)

            # then
            self.assertEqual(actual_visit, expected_visit)

    def test_from_release(self):
        release_input = {
            'id': hashutil.hash_to_bytes(
                'aad23fa492a0c5fed0708a6703be875448c86884'),
            'target': hashutil.hash_to_bytes(
                '5e46d564378afc44b31bb89f99d5675195fbdf67'),
            'target_type': 'revision',
            'date': {
                'timestamp': datetime.datetime(
                    2015, 1, 1, 22, 0, 0,
                    tzinfo=datetime.timezone.utc).timestamp(),
                'offset': 0,
                'negative_utc': False,
            },
            'author': {
                'name': b'author name',
                'fullname': b'Author Name author@email',
                'email': b'author@email',
            },
            'name': b'v0.0.1',
            'message': b'some comment on release',
            'synthetic': True,
        }

        expected_release = {
            'id': 'aad23fa492a0c5fed0708a6703be875448c86884',
            'target': '5e46d564378afc44b31bb89f99d5675195fbdf67',
            'target_type': 'revision',
            'date': '2015-01-01T22:00:00+00:00',
            'author': {
                'name': 'author name',
                'fullname': 'Author Name author@email',
                'email': 'author@email',
            },
            'name': 'v0.0.1',
            'message': 'some comment on release',
            'target_type': 'revision',
            'synthetic': True,
        }

        # when
        actual_release = converters.from_release(release_input)

        # then
        self.assertEqual(actual_release, expected_release)

    def test_from_release_no_revision(self):
        release_input = {
            'id': hashutil.hash_to_bytes(
                'b2171ee2bdf119cd99a7ec7eff32fa8013ef9a4e'),
            'target': None,
            'date': {
                'timestamp': datetime.datetime(
                    2016, 3, 2, 10, 0, 0,
                    tzinfo=datetime.timezone.utc).timestamp(),
                'offset': 0,
                'negative_utc': True,

            },
            'name': b'v0.1.1',
            'message': b'comment on release',
            'synthetic': False,
            'author': {
                'name': b'bob',
                'fullname': b'Bob bob@alice.net',
                'email': b'bob@alice.net',
            },
        }

        expected_release = {
            'id': 'b2171ee2bdf119cd99a7ec7eff32fa8013ef9a4e',
            'target': None,
            'date': '2016-03-02T10:00:00-00:00',
            'name': 'v0.1.1',
            'message': 'comment on release',
            'synthetic': False,
            'author': {
                'name': 'bob',
                'fullname': 'Bob bob@alice.net',
                'email': 'bob@alice.net',
            },
        }

        # when
        actual_release = converters.from_release(release_input)

        # then
        self.assertEqual(actual_release, expected_release)

    def test_from_revision(self):
        revision_input = {
            'id': hashutil.hash_to_bytes(
                '18d8be353ed3480476f032475e7c233eff7371d5'),
            'directory': hashutil.hash_to_bytes(
                '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6'),
            'author': {
                'name': b'Software Heritage',
                'fullname': b'robot robot@softwareheritage.org',
                'email': b'robot@softwareheritage.org',
            },
            'committer': {
                'name': b'Software Heritage',
                'fullname': b'robot robot@softwareheritage.org',
                'email': b'robot@softwareheritage.org',
            },
            'message': b'synthetic revision message',
            'date': {
                'timestamp': datetime.datetime(
                    2000, 1, 17, 11, 23, 54,
                    tzinfo=datetime.timezone.utc).timestamp(),
                'offset': 0,
                'negative_utc': False,
            },
            'committer_date': {
                'timestamp': datetime.datetime(
                    2000, 1, 17, 11, 23, 54,
                    tzinfo=datetime.timezone.utc).timestamp(),
                'offset': 0,
                'negative_utc': False,
            },
            'synthetic': True,
            'type': 'tar',
            'parents': [
                hashutil.hash_to_bytes(
                    '29d8be353ed3480476f032475e7c244eff7371d5'),
                hashutil.hash_to_bytes(
                    '30d8be353ed3480476f032475e7c244eff7371d5')
            ],
            'children': [
                hashutil.hash_to_bytes(
                    '123546353ed3480476f032475e7c244eff7371d5'),
            ],
            'metadata': {
                'extra_headers': [['gpgsig', b'some-signature']],
                'original_artifact': [{
                    'archive_type': 'tar',
                    'name': 'webbase-5.7.0.tar.gz',
                    'sha1': '147f73f369733d088b7a6fa9c4e0273dcd3c7ccd',
                    'sha1_git': '6a15ea8b881069adedf11feceec35588f2cfe8f1',
                    'sha256': '401d0df797110bea805d358b85bcc1ced29549d3d73f'
                    '309d36484e7edf7bb912',

                }]
            },
        }

        expected_revision = {
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'author': {
                'name': 'Software Heritage',
                'fullname': 'robot robot@softwareheritage.org',
                'email': 'robot@softwareheritage.org',
            },
            'committer': {
                'name': 'Software Heritage',
                'fullname': 'robot robot@softwareheritage.org',
                'email': 'robot@softwareheritage.org',
            },
            'message': 'synthetic revision message',
            'date': "2000-01-17T11:23:54+00:00",
            'committer_date': "2000-01-17T11:23:54+00:00",
            'children': [
                '123546353ed3480476f032475e7c244eff7371d5'
            ],
            'parents': [
                '29d8be353ed3480476f032475e7c244eff7371d5',
                '30d8be353ed3480476f032475e7c244eff7371d5'
            ],
            'type': 'tar',
            'synthetic': True,
            'metadata': {
                'extra_headers': [['gpgsig', 'some-signature']],
                'original_artifact': [{
                    'archive_type': 'tar',
                    'name': 'webbase-5.7.0.tar.gz',
                    'sha1': '147f73f369733d088b7a6fa9c4e0273dcd3c7ccd',
                    'sha1_git': '6a15ea8b881069adedf11feceec35588f2cfe8f1',
                    'sha256': '401d0df797110bea805d358b85bcc1ced29549d3d73f'
                    '309d36484e7edf7bb912'
                }]
            },
            'merge': True
        }

        # when
        actual_revision = converters.from_revision(revision_input)

        # then
        self.assertEqual(actual_revision, expected_revision)

    def test_from_revision_nomerge(self):
        revision_input = {
            'id': hashutil.hash_to_bytes(
                '18d8be353ed3480476f032475e7c233eff7371d5'),
            'parents': [
                hashutil.hash_to_bytes(
                    '29d8be353ed3480476f032475e7c244eff7371d5')
            ]
        }

        expected_revision = {
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'parents': [
                '29d8be353ed3480476f032475e7c244eff7371d5'
            ],
            'merge': False
        }

        # when
        actual_revision = converters.from_revision(revision_input)

        # then
        self.assertEqual(actual_revision, expected_revision)

    def test_from_revision_noparents(self):
        revision_input = {
            'id': hashutil.hash_to_bytes(
                '18d8be353ed3480476f032475e7c233eff7371d5'),
            'directory': hashutil.hash_to_bytes(
                '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6'),
            'author': {
                'name': b'Software Heritage',
                'fullname': b'robot robot@softwareheritage.org',
                'email': b'robot@softwareheritage.org',
            },
            'committer': {
                'name': b'Software Heritage',
                'fullname': b'robot robot@softwareheritage.org',
                'email': b'robot@softwareheritage.org',
            },
            'message': b'synthetic revision message',
            'date': {
                'timestamp': datetime.datetime(
                    2000, 1, 17, 11, 23, 54,
                    tzinfo=datetime.timezone.utc).timestamp(),
                'offset': 0,
                'negative_utc': False,
            },
            'committer_date': {
                'timestamp': datetime.datetime(
                    2000, 1, 17, 11, 23, 54,
                    tzinfo=datetime.timezone.utc).timestamp(),
                'offset': 0,
                'negative_utc': False,
            },
            'synthetic': True,
            'type': 'tar',
            'children': [
                hashutil.hash_to_bytes(
                    '123546353ed3480476f032475e7c244eff7371d5'),
            ],
            'metadata': {
                'original_artifact': [{
                    'archive_type': 'tar',
                    'name': 'webbase-5.7.0.tar.gz',
                    'sha1': '147f73f369733d088b7a6fa9c4e0273dcd3c7ccd',
                    'sha1_git': '6a15ea8b881069adedf11feceec35588f2cfe8f1',
                    'sha256': '401d0df797110bea805d358b85bcc1ced29549d3d73f'
                    '309d36484e7edf7bb912',

                }]
            },
        }

        expected_revision = {
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'author': {
                'name': 'Software Heritage',
                'fullname': 'robot robot@softwareheritage.org',
                'email': 'robot@softwareheritage.org',
            },
            'committer': {
                'name': 'Software Heritage',
                'fullname': 'robot robot@softwareheritage.org',
                'email': 'robot@softwareheritage.org',
            },
            'message': 'synthetic revision message',
            'date': "2000-01-17T11:23:54+00:00",
            'committer_date': "2000-01-17T11:23:54+00:00",
            'children': [
                '123546353ed3480476f032475e7c244eff7371d5'
            ],
            'type': 'tar',
            'synthetic': True,
            'metadata': {
                'original_artifact': [{
                    'archive_type': 'tar',
                    'name': 'webbase-5.7.0.tar.gz',
                    'sha1': '147f73f369733d088b7a6fa9c4e0273dcd3c7ccd',
                    'sha1_git': '6a15ea8b881069adedf11feceec35588f2cfe8f1',
                    'sha256': '401d0df797110bea805d358b85bcc1ced29549d3d73f'
                    '309d36484e7edf7bb912'
                }]
            }
        }

        # when
        actual_revision = converters.from_revision(revision_input)

        # then
        self.assertEqual(actual_revision, expected_revision)

    def test_from_revision_invalid(self):
        revision_input = {
            'id': hashutil.hash_to_bytes(
                '18d8be353ed3480476f032475e7c233eff7371d5'),
            'directory': hashutil.hash_to_bytes(
                '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6'),
            'author': {
                'name': b'Software Heritage',
                'fullname': b'robot robot@softwareheritage.org',
                'email': b'robot@softwareheritage.org',
            },
            'committer': {
                'name': b'Software Heritage',
                'fullname': b'robot robot@softwareheritage.org',
                'email': b'robot@softwareheritage.org',
            },
            'message': b'invalid message \xff',
            'date': {
                'timestamp': datetime.datetime(
                    2000, 1, 17, 11, 23, 54,
                    tzinfo=datetime.timezone.utc).timestamp(),
                'offset': 0,
                'negative_utc': False,
            },
            'committer_date': {
                'timestamp': datetime.datetime(
                    2000, 1, 17, 11, 23, 54,
                    tzinfo=datetime.timezone.utc).timestamp(),
                'offset': 0,
                'negative_utc': False,
            },
            'synthetic': True,
            'type': 'tar',
            'parents': [
                hashutil.hash_to_bytes(
                    '29d8be353ed3480476f032475e7c244eff7371d5'),
                hashutil.hash_to_bytes(
                    '30d8be353ed3480476f032475e7c244eff7371d5')
            ],
            'children': [
                hashutil.hash_to_bytes(
                    '123546353ed3480476f032475e7c244eff7371d5'),
            ],
            'metadata': {
                'original_artifact': [{
                    'archive_type': 'tar',
                    'name': 'webbase-5.7.0.tar.gz',
                    'sha1': '147f73f369733d088b7a6fa9c4e0273dcd3c7ccd',
                    'sha1_git': '6a15ea8b881069adedf11feceec35588f2cfe8f1',
                    'sha256': '401d0df797110bea805d358b85bcc1ced29549d3d73f'
                    '309d36484e7edf7bb912',

                }]
            },
        }

        expected_revision = {
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'author': {
                'name': 'Software Heritage',
                'fullname': 'robot robot@softwareheritage.org',
                'email': 'robot@softwareheritage.org',
            },
            'committer': {
                'name': 'Software Heritage',
                'fullname': 'robot robot@softwareheritage.org',
                'email': 'robot@softwareheritage.org',
            },
            'message': None,
            'message_decoding_failed': True,
            'date': "2000-01-17T11:23:54+00:00",
            'committer_date': "2000-01-17T11:23:54+00:00",
            'children': [
                '123546353ed3480476f032475e7c244eff7371d5'
            ],
            'parents': [
                '29d8be353ed3480476f032475e7c244eff7371d5',
                '30d8be353ed3480476f032475e7c244eff7371d5'
            ],
            'type': 'tar',
            'synthetic': True,
            'metadata': {
                'original_artifact': [{
                    'archive_type': 'tar',
                    'name': 'webbase-5.7.0.tar.gz',
                    'sha1': '147f73f369733d088b7a6fa9c4e0273dcd3c7ccd',
                    'sha1_git': '6a15ea8b881069adedf11feceec35588f2cfe8f1',
                    'sha256': '401d0df797110bea805d358b85bcc1ced29549d3d73f'
                    '309d36484e7edf7bb912'
                }]
            },
            'merge': True
        }

        # when
        actual_revision = converters.from_revision(revision_input)

        # then
        self.assertEqual(actual_revision, expected_revision)

    def test_from_content_none(self):
        self.assertIsNone(converters.from_content(None))

    def test_from_content(self):
        content_input = {
            'sha1': hashutil.hash_to_bytes(
                '5c6f0e2750f48fa0bd0c4cf5976ba0b9e02ebda5'),
            'sha256': hashutil.hash_to_bytes(
                '39007420ca5de7cb3cfc15196335507e'
                'e76c98930e7e0afa4d2747d3bf96c926'),
            'blake2s256': hashutil.hash_to_bytes(
                '49007420ca5de7cb3cfc15196335507e'
                'e76c98930e7e0afa4d2747d3bf96c926'),
            'sha1_git': hashutil.hash_to_bytes(
                '40e71b8614fcd89ccd17ca2b1d9e66c5b00a6d03'),
            'ctime': 'something-which-is-filtered-out',
            'data': b'data in bytes',
            'length': 10,
            'status': 'hidden',
        }

        # 'status' is filtered
        expected_content = {
            'checksums': {
                'sha1': '5c6f0e2750f48fa0bd0c4cf5976ba0b9e02ebda5',
                'sha256': '39007420ca5de7cb3cfc15196335507ee76c98'
                          '930e7e0afa4d2747d3bf96c926',
                'blake2s256': '49007420ca5de7cb3cfc15196335507ee7'
                              '6c98930e7e0afa4d2747d3bf96c926',
                'sha1_git': '40e71b8614fcd89ccd17ca2b1d9e66c5b00a6d03',
            },
            'data': b'data in bytes',
            'length': 10,
            'status': 'absent',
        }

        # when
        actual_content = converters.from_content(content_input)

        # then
        self.assertEqual(actual_content, expected_content)

    def test_from_person(self):
        person_input = {
            'id': 10,
            'anything': 'else',
            'name': b'bob',
            'fullname': b'bob bob@alice.net',
            'email': b'bob@foo.alice',
        }

        expected_person = {
            'id': 10,
            'anything': 'else',
            'name': 'bob',
            'fullname': 'bob bob@alice.net',
            'email': 'bob@foo.alice',
        }

        # when
        actual_person = converters.from_person(person_input)

        # then
        self.assertEqual(actual_person, expected_person)

    def test_from_directory_entries(self):
        dir_entries_input = {
            'sha1': hashutil.hash_to_bytes(
                '5c6f0e2750f48fa0bd0c4cf5976ba0b9e02ebda5'),
            'sha256': hashutil.hash_to_bytes(
                '39007420ca5de7cb3cfc15196335507e'
                'e76c98930e7e0afa4d2747d3bf96c926'),
            'sha1_git': hashutil.hash_to_bytes(
                '40e71b8614fcd89ccd17ca2b1d9e66c5b00a6d03'),
            'blake2s256': hashutil.hash_to_bytes(
                '685395c5dc57cada459364f0946d3dd45bad5fcbab'
                'c1048edb44380f1d31d0aa'),
            'target': hashutil.hash_to_bytes(
                '40e71b8614fcd89ccd17ca2b1d9e66c5b00a6d03'),
            'dir_id': hashutil.hash_to_bytes(
                '40e71b8614fcd89ccd17ca2b1d9e66c5b00a6d03'),
            'name': b'bob',
            'type': 10,
            'status': 'hidden',
        }

        expected_dir_entries = {
            'checksums': {
                'sha1': '5c6f0e2750f48fa0bd0c4cf5976ba0b9e02ebda5',
                'sha256': '39007420ca5de7cb3cfc15196335507ee76c98'
                          '930e7e0afa4d2747d3bf96c926',
                'sha1_git': '40e71b8614fcd89ccd17ca2b1d9e66c5b00a6d03',
                'blake2s256': '685395c5dc57cada459364f0946d3dd45bad5f'
                              'cbabc1048edb44380f1d31d0aa',
            },
            'target': '40e71b8614fcd89ccd17ca2b1d9e66c5b00a6d03',
            'dir_id': '40e71b8614fcd89ccd17ca2b1d9e66c5b00a6d03',
            'name': 'bob',
            'type': 10,
            'status': 'absent',
        }

        # when
        actual_dir_entries = converters.from_directory_entry(dir_entries_input)

        # then
        self.assertEqual(actual_dir_entries, expected_dir_entries)

    def test_from_filetype(self):
        content_filetype = {
            'id': hashutil.hash_to_bytes(
                '5c6f0e2750f48fa0bd0c4cf5976ba0b9e02ebda5'),
            'encoding': 'utf-8',
            'mimetype': 'text/plain',
        }

        expected_content_filetype = {
            'id': '5c6f0e2750f48fa0bd0c4cf5976ba0b9e02ebda5',
            'encoding': 'utf-8',
            'mimetype': 'text/plain',
        }

        # when
        actual_content_filetype = converters.from_filetype(content_filetype)

        # then
        self.assertEqual(actual_content_filetype, expected_content_filetype)



# Copyright (C) 2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from django.shortcuts import render
from django.views.decorators.clickjacking import xframe_options_exempt

# Current coverage list of the archive
# TODO: Retrieve that list dynamically instead of hardcoding it
_code_providers = [
    {
        'provider_url': 'https://www.debian.org/',
        'provider_logo': 'img/logos/debian.png',
        'provider_info': 'source packages from the Debian distribution '
                         '(continuously archived)',
    },
    {
        'provider_url': 'https://framagit.org/',
        'provider_logo': 'img/logos/framagit.png',
        'provider_info': 'public repositories from Framagit '
                         '(continuously archived)',
    },
    {
        'provider_url': 'https://github.com',
        'provider_logo': 'img/logos/github.png',
        'provider_info': 'public repositories from GitHub '
                         '(continuously archived)',
    },
    {
        'provider_url': 'https://gitlab.com',
        'provider_logo': 'img/logos/gitlab.svg',
        'provider_info': 'public repositories from GitLab '
                         '(continuously archived)',
    },
    {
        'provider_url': 'https://gitorious.org/',
        'provider_logo': 'img/logos/gitorious.png',
        'provider_info': 'public repositories from the former Gitorious code '
                         'hosting service',
    },
    {
        'provider_url': 'https://code.google.com/archive/',
        'provider_logo': 'img/logos/googlecode.png',
        'provider_info': 'public repositories from the former Google Code '
                         'project hosting service',
    },
    {
        'provider_url': 'https://www.gnu.org',
        'provider_logo': 'img/logos/gnu.png',
        'provider_info': 'releases from the GNU project (as of August 2015)',
    },
    {
        'provider_url': 'https://hal.archives-ouvertes.fr/',
        'provider_logo': 'img/logos/hal.png',
        'provider_info': 'scientific software source code deposited in the '
                         'open archive HAL'
    },
    {
        'provider_url': 'https://gitlab.inria.fr',
        'provider_logo': 'img/logos/inria.jpg',
        'provider_info': 'public repositories from Inria GitLab '
                         '(continuously archived)',
    },
    {
        'provider_url': 'https://pypi.org',
        'provider_logo': 'img/logos/pypi.svg',
        'provider_info': 'source packages from the Python Packaging Index '
                         '(continuously archived)',
    },
]


@xframe_options_exempt
def swh_coverage(request):
    return render(request, 'coverage.html', {'providers': _code_providers})



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

# flake8: noqa

from unittest.mock import patch

from django.utils.html import escape

from swh.web.common.exc import NotFoundExc
from swh.web.common.utils import (
    reverse, gen_path_info, format_utc_iso_date,
    parse_timestamp, get_swh_persistent_id
)
from swh.web.tests.testcase import WebTestCase

from .data.origin_test_data import (
    origin_info_test_data,
    origin_visits_test_data,
    stub_content_origin_info, stub_content_origin_visit_id,
    stub_content_origin_visit_unix_ts, stub_content_origin_visit_iso_date,
    stub_content_origin_branch,
    stub_content_origin_visits, stub_content_origin_snapshot,
    stub_origin_info, stub_visit_id,
    stub_origin_visits, stub_origin_snapshot,
    stub_origin_root_directory_entries, stub_origin_master_branch,
    stub_origin_root_directory_sha1, stub_origin_sub_directory_path,
    stub_origin_sub_directory_entries, stub_visit_unix_ts, stub_visit_iso_date
)

from .data.content_test_data import (
    stub_content_root_dir,
    stub_content_text_data,
    stub_content_text_path
)

stub_origin_info_no_type = dict(stub_origin_info)
stub_origin_info_no_type['type'] = None

def _to_snapshot_dict(branches=None, releases=None):
    snp = {'branches': {}}
    if branches:
        for b in branches:
            snp['branches'][b['name']] = {
                'target': b['revision'],
                'target_type': 'revision'
            }
    if releases:
        for r in releases:
            snp['branches'][r['branch_name']] =  {
                'target': r['id'],
                'target_type': 'release'
            }
    return snp

class SwhBrowseOriginTest(WebTestCase):

    @patch('swh.web.browse.utils.service')
    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.common.origin_visits.get_origin_visits')
    @patch('swh.web.browse.utils.get_origin_info')
    @patch('swh.web.browse.views.origin.get_origin_info')
    @patch('swh.web.browse.views.origin.get_origin_visits')
    @patch('swh.web.browse.views.origin.service')
    def test_origin_visits_browse(self, mock_service, mock_get_origin_visits,
                                  mock_get_origin_info, mock_get_origin_info_utils,
                                  mock_get_origin_visits_utils,
                                  mock_get_origin_visit_snapshot,
                                  mock_utils_service):
        mock_service.lookup_origin.return_value = origin_info_test_data
        mock_get_origin_info.return_value = origin_info_test_data
        mock_get_origin_info_utils.return_value = origin_info_test_data
        mock_get_origin_visits.return_value = origin_visits_test_data
        mock_get_origin_visits_utils.return_value = origin_visits_test_data
        mock_get_origin_visit_snapshot.return_value = stub_content_origin_snapshot
        mock_utils_service.lookup_snapshot_size.return_value = {
            'revision': len(stub_content_origin_snapshot[0]),
            'release': len(stub_content_origin_snapshot[1])
        }

        url = reverse('browse-origin-visits',
                      url_args={'origin_type': origin_info_test_data['type'],
                              'origin_url': origin_info_test_data['url']})
        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('origin-visits.html')

        url = reverse('browse-origin-visits',
                      url_args={'origin_url': origin_info_test_data['url']})
        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('origin-visits.html')

    def origin_content_view_helper(self, origin_info, origin_visits,
                                   origin_branches, origin_releases,
                                   origin_branch,
                                   root_dir_sha1, content_sha1, content_sha1_git,
                                   content_path, content_data,
                                   content_language,
                                   visit_id=None, timestamp=None):

        url_args = {'origin_type': origin_info['type'],
                    'origin_url': origin_info['url'],
                    'path': content_path}

        if not visit_id:
            visit_id = origin_visits[-1]['visit']

        query_params = {}

        if timestamp:
            url_args['timestamp'] = timestamp

        if visit_id:
            query_params['visit_id'] = visit_id

        url = reverse('browse-origin-content',
                      url_args=url_args,
                      query_params=query_params)

        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('content.html')

        self.assertContains(resp, '<code class="%s">' % content_language)
        self.assertContains(resp, escape(content_data))

        split_path = content_path.split('/')

        filename = split_path[-1]
        path = content_path.replace(filename, '')[:-1]

        path_info = gen_path_info(path)

        del url_args['path']

        if timestamp:
            url_args['timestamp'] = \
                format_utc_iso_date(parse_timestamp(timestamp).isoformat(),
                                    '%Y-%m-%dT%H:%M:%S')

        root_dir_url = reverse('browse-origin-directory',
                               url_args=url_args,
                               query_params=query_params)

        self.assertContains(resp, '<li class="swh-path">',
                            count=len(path_info)+1)


        self.assertContains(resp, '<a href="%s">%s</a>' %
                            (root_dir_url, root_dir_sha1[:7]))

        for p in path_info:
            url_args['path'] = p['path']
            dir_url = reverse('browse-origin-directory',
                              url_args=url_args,
                              query_params=query_params)
            self.assertContains(resp, '<a href="%s">%s</a>' %
                                (dir_url, p['name']))

        self.assertContains(resp, '<li>%s</li>' % filename)

        query_string = 'sha1_git:' + content_sha1

        url_raw = reverse('browse-content-raw',
                          url_args={'query_string': query_string},
                          query_params={'filename': filename})
        self.assertContains(resp, url_raw)

        del url_args['path']

        origin_branches_url = \
                reverse('browse-origin-branches',
                        url_args=url_args,
                        query_params=query_params)

        self.assertContains(resp, '<a href="%s">Branches (%s)</a>' %
            (origin_branches_url, len(origin_branches)))

        origin_releases_url = \
                reverse('browse-origin-releases',
                        url_args=url_args,
                        query_params=query_params)

        self.assertContains(resp, '<a href="%s">Releases (%s)</a>' %
            (origin_releases_url, len(origin_releases)))

        self.assertContains(resp, '<li class="swh-branch">',
                            count=len(origin_branches))

        url_args['path'] = content_path

        for branch in origin_branches:
            query_params['branch'] = branch['name']
            root_dir_branch_url = \
                reverse('browse-origin-content',
                        url_args=url_args,
                        query_params=query_params)

        self.assertContains(resp, '<a href="%s">' % root_dir_branch_url)

        self.assertContains(resp, '<li class="swh-release">',
                            count=len(origin_releases))

        query_params['branch'] = None
        for release in origin_releases:
            query_params['release'] = release['name']
            root_dir_release_url = \
                reverse('browse-origin-content',
                        url_args=url_args,
                        query_params=query_params)

            self.assertContains(resp, '<a href="%s">' % root_dir_release_url)

        del url_args['origin_type']

        url = reverse('browse-origin-content',
                      url_args=url_args,
                      query_params=query_params)

        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('content.html')

        swh_cnt_id = get_swh_persistent_id('content', content_sha1_git)
        swh_cnt_id_url = reverse('browse-swh-id',
                                 url_args={'swh_id': swh_cnt_id})
        self.assertContains(resp, swh_cnt_id)
        self.assertContains(resp, swh_cnt_id_url)

        self.assertContains(resp, 'swh-take-new-snapshot')

    @patch('swh.web.common.origin_visits.get_origin_visits')
    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.browse.views.utils.snapshot_context.service')
    @patch('swh.web.browse.utils.service')
    @patch('swh.web.browse.views.utils.snapshot_context.request_content')
    def test_origin_content_view(self, mock_request_content, mock_utils_service,
                                 mock_service, mock_get_origin_visit_snapshot,
                                 mock_get_origin_visits):

        stub_content_text_sha1 = stub_content_text_data['checksums']['sha1']
        stub_content_text_sha1_git = stub_content_text_data['checksums']['sha1_git']
        mock_get_origin_visits.return_value = stub_content_origin_visits
        mock_get_origin_visit_snapshot.return_value = stub_content_origin_snapshot
        mock_service.lookup_directory_with_path.return_value = \
            {'target': stub_content_text_sha1}
        mock_request_content.return_value = stub_content_text_data
        mock_utils_service.lookup_origin.return_value = stub_content_origin_info
        mock_utils_service.lookup_snapshot_size.return_value = {
            'revision': len(stub_content_origin_snapshot[0]),
            'release': len(stub_content_origin_snapshot[1])
        }

        self.origin_content_view_helper(stub_content_origin_info,
                                        stub_content_origin_visits,
                                        stub_content_origin_snapshot[0],
                                        stub_content_origin_snapshot[1],
                                        stub_content_origin_branch,
                                        stub_content_root_dir,
                                        stub_content_text_sha1,
                                        stub_content_text_sha1_git,
                                        stub_content_text_path,
                                        stub_content_text_data['raw_data'],
                                        'cpp')

        self.origin_content_view_helper(stub_content_origin_info,
                                        stub_content_origin_visits,
                                        stub_content_origin_snapshot[0],
                                        stub_content_origin_snapshot[1],
                                        stub_content_origin_branch,
                                        stub_content_root_dir,
                                        stub_content_text_sha1,
                                        stub_content_text_sha1_git,
                                        stub_content_text_path,
                                        stub_content_text_data['raw_data'],
                                        'cpp',
                                        visit_id=stub_content_origin_visit_id)

        self.origin_content_view_helper(stub_content_origin_info,
                                        stub_content_origin_visits,
                                        stub_content_origin_snapshot[0],
                                        stub_content_origin_snapshot[1],
                                        stub_content_origin_branch,
                                        stub_content_root_dir,
                                        stub_content_text_sha1,
                                        stub_content_text_sha1_git,
                                        stub_content_text_path,
                                        stub_content_text_data['raw_data'],
                                        'cpp',
                                        timestamp=stub_content_origin_visit_unix_ts)

        self.origin_content_view_helper(stub_content_origin_info,
                                        stub_content_origin_visits,
                                        stub_content_origin_snapshot[0],
                                        stub_content_origin_snapshot[1],
                                        stub_content_origin_branch,
                                        stub_content_root_dir,
                                        stub_content_text_sha1,
                                        stub_content_text_sha1_git,
                                        stub_content_text_path,
                                        stub_content_text_data['raw_data'],
                                        'cpp',
                                        timestamp=stub_content_origin_visit_iso_date)

    def origin_directory_view_helper(self, origin_info, origin_visits,
                                     origin_branches, origin_releases, origin_branch,
                                     root_directory_sha1, directory_entries,
                                     visit_id=None, timestamp=None, path=None):

        dirs = [e for e in directory_entries
                if e['type'] in ('dir', 'rev')]
        files = [e for e in directory_entries
                 if e['type'] == 'file']

        if not visit_id:
            visit_id = origin_visits[-1]['visit']

        url_args = {'origin_url': origin_info['url']}

        query_params = {}

        if timestamp:
            url_args['timestamp'] = timestamp
        else:
            query_params['visit_id'] = visit_id

        if path:
            url_args['path'] = path

        url = reverse('browse-origin-directory',
                      url_args=url_args,
                      query_params=query_params)

        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('directory.html')

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('directory.html')


        self.assertContains(resp, '<td class="swh-directory">',
                            count=len(dirs))
        self.assertContains(resp, '<td class="swh-content">',
                            count=len(files))

        if timestamp:
            url_args['timestamp'] = \
                format_utc_iso_date(parse_timestamp(timestamp).isoformat(),
                                    '%Y-%m-%dT%H:%M:%S')

        for d in dirs:
            if d['type'] == 'rev':
                dir_url = reverse('browse-revision',
                                  url_args={'sha1_git': d['target']})
            else:
                dir_path = d['name']
                if path:
                    dir_path = "%s/%s" % (path, d['name'])
                dir_url_args = dict(url_args)
                dir_url_args['path'] = dir_path
                dir_url = reverse('browse-origin-directory',
                                url_args=dir_url_args,
                                query_params=query_params)
            self.assertContains(resp, dir_url)

        for f in files:
            file_path = f['name']
            if path:
                file_path = "%s/%s" % (path, f['name'])
            file_url_args = dict(url_args)
            file_url_args['path'] = file_path
            file_url = reverse('browse-origin-content',
                               url_args=file_url_args,
                               query_params=query_params)
            self.assertContains(resp, file_url)

        if 'path' in url_args:
            del url_args['path']

        root_dir_branch_url = \
            reverse('browse-origin-directory',
                    url_args=url_args,
                    query_params=query_params)

        nb_bc_paths = 1
        if path:
            nb_bc_paths = len(path.split('/')) + 1

        self.assertContains(resp, '<li class="swh-path">', count=nb_bc_paths)
        self.assertContains(resp, '<a href="%s">%s</a>' %
                                  (root_dir_branch_url,
                                   root_directory_sha1[:7]))

        origin_branches_url = \
                reverse('browse-origin-branches',
                        url_args=url_args,
                        query_params=query_params)

        self.assertContains(resp, '<a href="%s">Branches (%s)</a>' %
            (origin_branches_url, len(origin_branches)))

        origin_releases_url = \
                reverse('browse-origin-releases',
                        url_args=url_args,
                        query_params=query_params)

        self.assertContains(resp, '<a href="%s">Releases (%s)</a>' %
            (origin_releases_url, len(origin_releases)))

        if path:
            url_args['path'] = path

        self.assertContains(resp, '<li class="swh-branch">',
                            count=len(origin_branches))

        for branch in origin_branches:
            query_params['branch'] = branch['name']
            root_dir_branch_url = \
                reverse('browse-origin-directory',
                        url_args=url_args,
                        query_params=query_params)

            self.assertContains(resp, '<a href="%s">' % root_dir_branch_url)

        self.assertContains(resp, '<li class="swh-release">',
                            count=len(origin_releases))

        query_params['branch'] = None
        for release in origin_releases:
            query_params['release'] = release['name']
            root_dir_release_url = \
                reverse('browse-origin-directory',
                        url_args=url_args,
                        query_params=query_params)

            self.assertContains(resp, '<a href="%s">' % root_dir_release_url)

        self.assertContains(resp, 'vault-cook-directory')
        self.assertContains(resp, 'vault-cook-revision')

        swh_dir_id = get_swh_persistent_id('directory', directory_entries[0]['dir_id']) # noqa
        swh_dir_id_url = reverse('browse-swh-id',
                                 url_args={'swh_id': swh_dir_id})
        self.assertContains(resp, swh_dir_id)
        self.assertContains(resp, swh_dir_id_url)

        self.assertContains(resp, 'swh-take-new-snapshot')


    @patch('swh.web.common.origin_visits.get_origin_visits')
    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.browse.utils.service')
    @patch('swh.web.browse.views.origin.service')
    def test_origin_root_directory_view(self, mock_origin_service,
                                        mock_utils_service,
                                        mock_get_origin_visit_snapshot,
                                        mock_get_origin_visits):

        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.return_value = stub_origin_snapshot
        mock_utils_service.lookup_directory.return_value = \
            stub_origin_root_directory_entries
        mock_utils_service.lookup_origin.return_value = stub_origin_info
        mock_utils_service.lookup_snapshot_size.return_value = {
            'revision': len(stub_origin_snapshot[0]),
            'release': len(stub_origin_snapshot[1])
        }

        self.origin_directory_view_helper(stub_origin_info, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_root_directory_entries)

        self.origin_directory_view_helper(stub_origin_info, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_root_directory_entries,
                                          visit_id=stub_visit_id)

        self.origin_directory_view_helper(stub_origin_info, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_root_directory_entries,
                                          timestamp=stub_visit_unix_ts)

        self.origin_directory_view_helper(stub_origin_info, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_root_directory_entries,
                                          timestamp=stub_visit_iso_date)

        self.origin_directory_view_helper(stub_origin_info_no_type, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_root_directory_entries)

        self.origin_directory_view_helper(stub_origin_info_no_type, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_root_directory_entries,
                                          visit_id=stub_visit_id)

        self.origin_directory_view_helper(stub_origin_info_no_type, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_root_directory_entries,
                                          timestamp=stub_visit_unix_ts)

        self.origin_directory_view_helper(stub_origin_info_no_type, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_root_directory_entries,
                                          timestamp=stub_visit_iso_date)

    @patch('swh.web.common.origin_visits.get_origin_visits')
    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.browse.utils.service')
    @patch('swh.web.browse.views.utils.snapshot_context.service')
    def test_origin_sub_directory_view(self, mock_origin_service,
                                       mock_utils_service,
                                       mock_get_origin_visit_snapshot,
                                       mock_get_origin_visits):

        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.return_value = stub_origin_snapshot
        mock_utils_service.lookup_directory.return_value = \
            stub_origin_sub_directory_entries
        mock_origin_service.lookup_directory_with_path.return_value = \
            {'target': stub_origin_sub_directory_entries[0]['dir_id'],
             'type' : 'dir'}
        mock_utils_service.lookup_origin.return_value = stub_origin_info
        mock_utils_service.lookup_snapshot_size.return_value = {
            'revision': len(stub_origin_snapshot[0]),
            'release': len(stub_origin_snapshot[1])
        }

        self.origin_directory_view_helper(stub_origin_info, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_sub_directory_entries,
                                          path=stub_origin_sub_directory_path)

        self.origin_directory_view_helper(stub_origin_info, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_sub_directory_entries,
                                          visit_id=stub_visit_id,
                                          path=stub_origin_sub_directory_path)

        self.origin_directory_view_helper(stub_origin_info, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_sub_directory_entries,
                                          timestamp=stub_visit_unix_ts,
                                          path=stub_origin_sub_directory_path)

        self.origin_directory_view_helper(stub_origin_info, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_sub_directory_entries,
                                          timestamp=stub_visit_iso_date,
                                          path=stub_origin_sub_directory_path)

        self.origin_directory_view_helper(stub_origin_info_no_type, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_sub_directory_entries,
                                          path=stub_origin_sub_directory_path)

        self.origin_directory_view_helper(stub_origin_info_no_type, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_sub_directory_entries,
                                          visit_id=stub_visit_id,
                                          path=stub_origin_sub_directory_path)

        self.origin_directory_view_helper(stub_origin_info_no_type, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_sub_directory_entries,
                                          timestamp=stub_visit_unix_ts,
                                          path=stub_origin_sub_directory_path)

        self.origin_directory_view_helper(stub_origin_info_no_type, stub_origin_visits,
                                          stub_origin_snapshot[0],
                                          stub_origin_snapshot[1],
                                          stub_origin_master_branch,
                                          stub_origin_root_directory_sha1,
                                          stub_origin_sub_directory_entries,
                                          timestamp=stub_visit_iso_date,
                                          path=stub_origin_sub_directory_path)

    @patch('swh.web.browse.views.utils.snapshot_context.request_content')
    @patch('swh.web.common.origin_visits.get_origin_visits')
    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.browse.utils.service')
    @patch('swh.web.browse.views.origin.service')
    @patch('swh.web.browse.views.utils.snapshot_context.service')
    @patch('swh.web.browse.views.origin.get_origin_info')
    def test_origin_request_errors(self, mock_get_origin_info,
                                   mock_snapshot_service,
                                   mock_origin_service,
                                   mock_utils_service,
                                   mock_get_origin_visit_snapshot,
                                   mock_get_origin_visits,
                                   mock_request_content):

        mock_get_origin_info.side_effect = \
            NotFoundExc('origin not found')
        url = reverse('browse-origin-visits',
                      url_args={'origin_type': 'foo',
                                'origin_url': 'bar'})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertContains(resp, 'origin not found', status_code=404)

        mock_utils_service.lookup_origin.side_effect = None
        mock_utils_service.lookup_origin.return_value = origin_info_test_data
        mock_get_origin_visits.return_value = []
        url = reverse('browse-origin-directory',
                      url_args={'origin_type': 'foo',
                                'origin_url': 'bar'})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertContains(resp, "No visit", status_code=404)

        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.side_effect = \
            NotFoundExc('visit not found')
        url = reverse('browse-origin-directory',
                      url_args={'origin_type': 'foo',
                                'origin_url': 'bar'},
                      query_params={'visit_id': len(stub_origin_visits)+1})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertRegex(resp.content.decode('utf-8'), 'Visit.*not found')

        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.side_effect = None
        mock_get_origin_visit_snapshot.return_value = stub_origin_snapshot
        mock_utils_service.lookup_snapshot_size.return_value = {
            'revision': len(stub_origin_snapshot[0]),
            'release': len(stub_origin_snapshot[1])
        }
        mock_utils_service.lookup_directory.side_effect = \
            NotFoundExc('Directory not found')
        url = reverse('browse-origin-directory',
                      url_args={'origin_type': 'foo',
                                'origin_url': 'bar'})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertContains(resp, 'Directory not found', status_code=404)

        with patch('swh.web.browse.views.utils.snapshot_context.get_snapshot_context') \
                as mock_get_snapshot_context:
            mock_get_snapshot_context.side_effect = \
                NotFoundExc('Snapshot not found')
            url = reverse('browse-origin-directory',
                          url_args={'origin_type': 'foo',
                                    'origin_url': 'bar'})
            resp = self.client.get(url)
            self.assertEqual(resp.status_code, 404)
            self.assertTemplateUsed('error.html')
            self.assertContains(resp, 'Snapshot not found', status_code=404)

        mock_origin_service.lookup_origin.side_effect = None
        mock_origin_service.lookup_origin.return_value = origin_info_test_data
        mock_get_origin_visits.return_value = []
        url = reverse('browse-origin-content',
                      url_args={'origin_type': 'foo',
                                'origin_url': 'bar',
                                'path': 'foo'})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertContains(resp, "No visit", status_code=404)

        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.side_effect = \
            NotFoundExc('visit not found')
        url = reverse('browse-origin-content',
                      url_args={'origin_type': 'foo',
                                'origin_url': 'bar',
                                'path': 'foo'},
                      query_params={'visit_id': len(stub_origin_visits)+1})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertRegex(resp.content.decode('utf-8'), 'Visit.*not found')

        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.side_effect = None
        mock_get_origin_visit_snapshot.return_value = ([], [])
        url = reverse('browse-origin-content',
                      url_args={'origin_type': 'foo',
                              'origin_url': 'bar',
                              'path': 'baz'})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertRegex(resp.content.decode('utf-8'),
                         'Origin.*has an empty list of branches')

        mock_get_origin_visit_snapshot.return_value = stub_origin_snapshot
        mock_snapshot_service.lookup_directory_with_path.return_value = \
            {'target': stub_content_text_data['checksums']['sha1']}
        mock_request_content.side_effect = \
            NotFoundExc('Content not found')
        url = reverse('browse-origin-content',
                      url_args={'origin_type': 'foo',
                              'origin_url': 'bar',
                              'path': 'baz'})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertContains(resp, 'Content not found', status_code=404)


    @patch('swh.web.common.origin_visits.get_origin_visits')
    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.browse.utils.service')
    def test_origin_empty_snapshot(self, mock_utils_service,
                                   mock_get_origin_visit_snapshot,
                                   mock_get_origin_visits):

        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.return_value = ([], [])
        mock_utils_service.lookup_snapshot_size.return_value = {
            'revision': 0,
            'release': 0
        }
        url = reverse('browse-origin-directory',
                      url_args={'origin_type': 'foo',
                              'origin_url': 'bar'})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('content.html')
        self.assertRegex(resp.content.decode('utf-8'), 'snapshot.*is empty')

    def origin_branches_helper(self, origin_info, origin_snapshot):
        url_args = {'origin_type': origin_info['type'],
                    'origin_url': origin_info['url']}

        url = reverse('browse-origin-branches',
                      url_args=url_args)

        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('branches.html')

        origin_branches = origin_snapshot[0]
        origin_releases = origin_snapshot[1]

        origin_branches_url = \
                reverse('browse-origin-branches',
                        url_args=url_args)

        self.assertContains(resp, '<a href="%s">Branches (%s)</a>' %
            (origin_branches_url, len(origin_branches)))

        origin_releases_url = \
                reverse('browse-origin-releases',
                        url_args=url_args)

        self.assertContains(resp, '<a href="%s">Releases (%s)</a>' %
            (origin_releases_url, len(origin_releases)))

        self.assertContains(resp, '<tr class="swh-branch-entry',
                            count=len(origin_branches))

        for branch in origin_branches:
            browse_branch_url = reverse('browse-origin-directory',
                                        url_args={'origin_type': origin_info['type'],
                                                'origin_url': origin_info['url']},
                                        query_params={'branch': branch['name']})
            self.assertContains(resp, '<a href="%s">' % escape(browse_branch_url))

            browse_revision_url = reverse('browse-revision',
                                          url_args={'sha1_git': branch['revision']},
                                          query_params={'origin_type': origin_info['type'],
                                                        'origin': origin_info['url']})
            self.assertContains(resp, '<a href="%s">' % escape(browse_revision_url))


    @patch('swh.web.browse.views.utils.snapshot_context.process_snapshot_branches')
    @patch('swh.web.browse.views.utils.snapshot_context.service')
    @patch('swh.web.common.origin_visits.get_origin_visits')
    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.browse.utils.service')
    @patch('swh.web.browse.views.origin.service')
    def test_origin_branches(self, mock_origin_service,
                             mock_utils_service,
                             mock_get_origin_visit_snapshot,
                             mock_get_origin_visits,
                             mock_snp_ctx_service,
                             mock_snp_ctx_process_branches):
        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.return_value = stub_origin_snapshot
        mock_utils_service.lookup_origin.return_value = stub_origin_info
        mock_utils_service.lookup_snapshot_size.return_value = \
            {'revision': len(stub_origin_snapshot[0]), 'release': len(stub_origin_snapshot[1])}
        mock_snp_ctx_service.lookup_snapshot.return_value = \
            _to_snapshot_dict(branches=stub_origin_snapshot[0])
        mock_snp_ctx_process_branches.return_value = stub_origin_snapshot

        self.origin_branches_helper(stub_origin_info, stub_origin_snapshot)

        self.origin_branches_helper(stub_origin_info_no_type, stub_origin_snapshot)


    def origin_releases_helper(self, origin_info, origin_snapshot):
        url_args = {'origin_type': origin_info['type'],
                    'origin_url': origin_info['url']}

        url = reverse('browse-origin-releases',
                      url_args=url_args)

        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('releases.html')

        origin_branches = origin_snapshot[0]
        origin_releases = origin_snapshot[1]

        origin_branches_url = \
                reverse('browse-origin-branches',
                        url_args=url_args)

        self.assertContains(resp, '<a href="%s">Branches (%s)</a>' %
            (origin_branches_url, len(origin_branches)))

        origin_releases_url = \
                reverse('browse-origin-releases',
                        url_args=url_args)

        self.assertContains(resp, '<a href="%s">Releases (%s)</a>' %
            (origin_releases_url, len(origin_releases)))

        self.assertContains(resp, '<tr class="swh-release-entry',
                            count=len(origin_releases))

        for release in origin_releases:
            browse_release_url = reverse('browse-release',
                                         url_args={'sha1_git': release['id']},
                                         query_params={'origin': origin_info['url']})
            browse_revision_url = reverse('browse-revision',
                                          url_args={'sha1_git': release['target']},
                                          query_params={'origin': origin_info['url']})

            self.assertContains(resp, '<a href="%s">' % escape(browse_release_url))
            self.assertContains(resp, '<a href="%s">' % escape(browse_revision_url))


    @patch('swh.web.browse.views.utils.snapshot_context.process_snapshot_branches')
    @patch('swh.web.browse.views.utils.snapshot_context.service')
    @patch('swh.web.common.origin_visits.get_origin_visits')
    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.browse.utils.service')
    @patch('swh.web.browse.views.origin.service')
    def test_origin_releases(self, mock_origin_service,
                             mock_utils_service,
                             mock_get_origin_visit_snapshot,
                             mock_get_origin_visits,
                             mock_snp_ctx_service,
                             mock_snp_ctx_process_branches):
        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.return_value = stub_origin_snapshot
        mock_utils_service.lookup_origin.return_value = stub_origin_info
        mock_utils_service.lookup_snapshot_size.return_value = \
            {'revision': len(stub_origin_snapshot[0]), 'release': len(stub_origin_snapshot[1])}
        mock_snp_ctx_service.lookup_snapshot.return_value = \
            _to_snapshot_dict(releases=stub_origin_snapshot[1])
        mock_snp_ctx_process_branches.return_value = stub_origin_snapshot

        self.origin_releases_helper(stub_origin_info, stub_origin_snapshot)
        self.origin_releases_helper(stub_origin_info_no_type, stub_origin_snapshot)




# Copyright (C) 2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

# flake8: noqa

stub_release = \
    {'target': '69c0db5050f623e8895b72dfe970392b1f9a0e2e',
     'author': {'name': 'Ned Deily',
                'fullname': 'Ned Deily <nad@python.org>',
                'email': 'nad@python.org',
                'id': 8318464},
     'id': '208f61cc7a5dbc9879ae6e5c2f95891e270f09ef',
     'name': 'v3.6.1',
     'date': '2017-03-21T02:46:28-04:00',
     'message': 'Tag v3.6.1\n-----BEGIN PGP SIGNATURE-----\n\niQIcBAABCgAGBQJY0MxgAAoJEC00fqaqZUIdkZ0QAJw9PR++cbpS3Pt8QrmgS+xG\nPxrZ1yPPNPNSfbmRLWOlHJ0nBzFPVXUWdrqnevmZVRghyrc78sjuBL8QczYsum22\n1B6X/63vX3dI9yj8FR5nldEYPBMOOD6ryObWoKMeqyQT3LhAqxIU/9oqAsbx+ZYw\nrXmRTuypenmZabq3yIv2hORMFgcS7JZFuVb181b0Cihji/7l+WRI9hkGO8POBeFq\ntfJ16beH8hbbDw/+MLpwJifsALWsQOqnWt2/C8tJeHtMX+FLuJflwcIwotv73E22\nulmpXNwTNxnK5l5/C9JC6kr5nN9VJatVpSpe6dftAmTy16O5OrADtePZYxOZ7S3X\n6ipOaiKl3s/2oykkmasxPeaVXllbWgd2UGqIBlAUxM6rVD/4DyVDUHqbDotQD8Kz\nZ8nSFxou1ZdRTSlC26ToGCNc+B6bqv9GTC1hph/ijJkhvXfIC9X1fc/uO1wrV+wB\ni2dxXKh1mQCXuogNAx6rv7gPaXbPgDHob7Tlvo5Ddhr7rQoAaMjceGfUMOTORSqO\nR4ssE6yyNASQtMjW+Y5WeVEgtX7ttGKBsgD0PsrZTCjnZfJkFtZGUyfkdwNzLK8v\nRBqi1r+tEuR5tpin4h+erdlVjeMhVMQZOhBYmxY2Ge70PMVrOz4KaFY1GD+aaxt7\n+PfOKUxMYGKvogv7gD/3\n=Peec\n-----END PGP SIGNATURE-----\n',
     'target_type': 'revision',
     'synthetic': False}



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from swh.core import config
from swh.storage import get_storage
from swh.indexer.storage import get_indexer_storage
from swh.vault.api.client import RemoteVaultClient
from swh.scheduler import get_scheduler


DEFAULT_CONFIG = {
    'allowed_hosts': ('list', []),
    'storage': ('dict', {
        'cls': 'remote',
        'args': {
            'url': 'http://127.0.0.1:5002/',
            'timeout': 10,
        },
    }),
    'indexer_storage': ('dict', {
        'cls': 'remote',
        'args': {
            'url': 'http://127.0.0.1:5007/',
            'timeout': 1,
        }
    }),
    'vault': ('string', 'http://127.0.0.1:5005/'),
    'log_dir': ('string', '/tmp/swh/log'),
    'debug': ('bool', False),
    'serve_assets': ('bool', False),
    'host': ('string', '127.0.0.1'),
    'port': ('int', 5004),
    'secret_key': ('string', 'development key'),
    # do not display code highlighting for content > 1MB
    'content_display_max_size': ('int', 1024 * 1024),
    'snapshot_content_max_size': ('int', 1000),
    'throttling': ('dict', {
        'cache_uri': None,  # production: memcached as cache (127.0.0.1:11211)
                            # development: in-memory cache so None
        'scopes': {
            'swh_api': {
                'limiter_rate': {
                    'default': '120/h'
                },
                'exempted_networks': ['127.0.0.0/8']
            },
            'swh_vault_cooking': {
                'limiter_rate': {
                    'default': '120/h',
                    'GET': '60/m'
                },
                'exempted_networks': ['127.0.0.0/8']
            },
            'swh_save_origin': {
                'limiter_rate': {
                    'default': '120/h',
                    'POST': '10/h'
                },
                'exempted_networks': ['127.0.0.0/8']
            }
        }
    }),
    'scheduler': ('dict', {
        'cls': 'remote',
        'args': {
            'url': 'http://localhost:5008/'
        }
    }),
    'grecaptcha': ('dict', {
        'activated': True,
        'validation_url': 'https://www.google.com/recaptcha/api/siteverify',
        'site_key': '',
        'private_key': ''
    }),
    'production_db': ('string', '/var/lib/swh/web.sqlite3'),
    'deposit': ('dict', {
        'private_api_url': 'https://deposit.softwareheritage.org/1/private/',
        'private_api_user': 'swhworker',
        'private_api_password': ''
    })
}

swhweb_config = {}


def get_config(config_file='web/web'):
    """Read the configuration file `config_file`, update the app with
       parameters (secret_key, conf) and return the parsed configuration as a
       dict. If no configuration file is provided, return a default
       configuration."""

    if not swhweb_config:
        cfg = config.load_named_config(config_file, DEFAULT_CONFIG)
        swhweb_config.update(cfg)
        config.prepare_folders(swhweb_config, 'log_dir')
        swhweb_config['storage'] = get_storage(**swhweb_config['storage'])
        swhweb_config['vault'] = RemoteVaultClient(swhweb_config['vault'])
        swhweb_config['indexer_storage'] = \
            get_indexer_storage(**swhweb_config['indexer_storage'])
        swhweb_config['scheduler'] = get_scheduler(**swhweb_config['scheduler']) # noqa
    return swhweb_config


def storage():
    """Return the current application's storage.

    """
    return get_config()['storage']


def vault():
    """Return the current application's vault.

    """
    return get_config()['vault']


def indexer_storage():
    """Return the current application's indexer storage.

    """
    return get_config()['indexer_storage']


def scheduler():
    """Return the current application's scheduler.

    """
    return get_config()['scheduler']



# Copyright (C) 2015-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from swh.web.common import service
from swh.web.api import utils
from swh.web.api.apidoc import api_doc
from swh.web.api.apiurls import api_route
from swh.web.api.views.utils import api_lookup


@api_route(r'/release/(?P<sha1_git>[0-9a-f]+)/', 'api-release')
@api_doc('/release/')
def api_release(request, sha1_git):
    """
    .. http:get:: /api/1/release/(sha1_git)/

        Get information about a release in the archive.
        Releases are identified by **sha1** checksums, compatible with Git tag identifiers.
        See :func:`swh.model.identifiers.release_identifier` in our data model module for details
        about how they are computed.

        :param string sha1_git: hexadecimal representation of the release **sha1_git** identifier

        :reqheader Accept: the requested response content type,
            either ``application/json`` (default) or ``application/yaml``
        :resheader Content-Type: this depends on :http:header:`Accept` header of request

        :>json object author: information about the author of the release
        :>json string author_url: link to :http:get:`/api/1/person/(person_id)/` to get
            information about the author of the release
        :>json string date: ISO representation of the release date (in UTC)
        :>json string id: the release unique identifier
        :>json string message: the message associated to the release
        :>json string name: the name of the release
        :>json string target: the target identifier of the release
        :>json string target_type: the type of the target, can be either **release**,
            **revision**, **content**, **directory**
        :>json string target_url: a link to the adequate api url based on the target type

        **Allowed HTTP Methods:** :http:method:`get`, :http:method:`head`, :http:method:`options`

        :statuscode 200: no error
        :statuscode 400: an invalid **sha1_git** value has been provided
        :statuscode 404: requested release can not be found in the archive

        **Example:**

        .. parsed-literal::

            :swh_web_api:`release/208f61cc7a5dbc9879ae6e5c2f95891e270f09ef/`
    """ # noqa
    error_msg = 'Release with sha1_git %s not found.' % sha1_git
    return api_lookup(
        service.lookup_release, sha1_git,
        notfound_msg=error_msg,
        enrich_fn=utils.enrich_release)



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

# flake8: noqa

"""
Django development settings for swh-web.
"""

import os

from .common import *

MIDDLEWARE += ['swh.web.common.middlewares.HtmlPrettifyMiddleware']

from django.core.cache import cache
cache.clear()
